notification

package
v0.86.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: May 10, 2026 License: Apache-2.0 Imports: 9 Imported by: 0

Documentation

Index

Constants

const (
	Info    level = iota // Informational messages
	Success              // Successful operations
	Warning              // Warning messages indicating potential issues
	Failure              // Error messages indicating failure of operations
)

Variables

var ErrNotifyFailed = errors.New("request to apprise failed")

ErrNotifyFailed is returned when the Apprise request fails due to invalid notify URLs or unreachable service.

Functions

func GetRevision added in v0.32.1

func GetRevision(reference, commitSHA string) string

func Send

func Send(level level, title, message string, metadata Metadata) error

Send sends a notification using the Apprise service based on the provided configuration and parameters.

func SetAppriseConfig

func SetAppriseConfig(apiURL, notifyUrls, notifyLevel string)

SetAppriseConfig sets the configuration for the Apprise notification service.

Types

type Metadata added in v0.32.1

type Metadata struct {
	Repository          string
	Stack               string
	Revision            string
	JobID               string
	TraceID             string
	ReconciliationEvent string
	AffectedActorKind   string
	AffectedActorID     string
	AffectedActorName   string
}

Jump to